import asyncio
import csv
import os
from guess import Guess

CSV_FILE_PATH = "../../data/guess/guess.csv"

# Define RPS Constant Experiment class
class GuessExperiment:
    def __init__(self):
        self.models = ["llama3", "deepseek-r1",] #  You can also add "llama3" "deepseek-r1" "gpt-4.5-preview-2025-02-27", "mistral-small", "llama3.3:latest", "mixtral:8x7b", "deepseek-r1:7b"
        self.opponent_strategies = {
            "always_rock": lambda history: "Rock",
            "always_paper": lambda history: "Paper",
            "always_scissors": lambda history: "Scissos",
            "R-P": self.loop_R_P,
            "P-S": self.loop_P_S,
            "S-R": self.loop_S_R,
            "R-P-S": self.loop_R_P_S
        }
        self.temperature = 0.7
        self.rounds = 10
        self.num_games_per_config = 10# 30
        self.strategy = True
        self.initialize_csv()

    @staticmethod
    def loop_R_P(history):
        """Alternates between Rock and Paper (R-P)"""
        if len(history) % 2 == 0:
            return "Rock"
        else:
            return "Paper"


    @staticmethod
    def loop_P_S(history):
        """Alternates between Paper and Scissors (P-S)"""
        if len(history) % 2 == 0:
            return "Paper"
        else:
            return "Scissors"

    @staticmethod
    def loop_S_R(history):
        """Alternates between Scissors and Rock (S-R)"""
        if len(history) % 2 == 0:
            return "Scissors"
        else:
            return "Rock"

    @staticmethod
    def loop_R_P_S(history):
        """Alternates between Rock, Paper, and Scissors (R-P-S)"""
        strategies = ["Rock", "Paper", "Scissors"]
        return strategies[len(history) % 3]

    def initialize_csv(self):
        if not os.path.exists(CSV_FILE_PATH):
            os.makedirs(os.path.dirname(CSV_FILE_PATH), exist_ok=True)
            with open(CSV_FILE_PATH, mode="w", newline="") as file:
                writer = csv.writer(file)
                writer.writerow(["idGame", "model", "opponentStrategy", "idRound", "playerPrediction", "opponentMove", "outcomeRound", "currentPlayerScoreGame", "reasoning"])


    def sanitize_reasoning(self, reasoning: str) -> str:
        sanitized = reasoning.replace('"', '""')
        sanitized = sanitized.replace('\n', ' ').replace('\r', '')
        if sanitized and sanitized[0] in ('=', '+', '-', '@'):
            sanitized = "'" + sanitized
        return f'"{sanitized}"'

    def log_to_csv(self, game_id, model, opponent_strategy, round_id, agent_prediction, opponent_move, outcome, player_score_game, reasoning):
        sanitized_reasoning = self.sanitize_reasoning(reasoning)
        with open(CSV_FILE_PATH, mode="a", newline="") as file:
            writer = csv.writer(file)
            model_type = model+" strategy" if self.strategy else model
            writer.writerow([game_id, model_type, opponent_strategy, round_id, agent_prediction, opponent_move, outcome, player_score_game, sanitized_reasoning])

    async def run_experiment(self):
        """Runs the experiment for all configurations."""
        tasks = []
        game_id = 1
        for model in self.models:
            for strategy_name, strategy_fn in self.opponent_strategies.items():
                for _ in range(self.num_games_per_config):
                    tasks.append(self.run_game(model, strategy_name, strategy_fn, game_id))
                    game_id += 1
        await asyncio.gather(*tasks)


    async def run_game(self, model, opponent_strategy_name, opponent_strategy_fn, game_id):
        game = Guess(model=model, temperature=self.temperature, game_id=game_id, opponent_strategy_fn=opponent_strategy_fn, strategy = self.strategy)
        for i in range(1, self.rounds + 1):
            round_data = await game.play_round(i)
            self.log_to_csv(game_id, model, opponent_strategy_name, i, round_data["Agent Prediction"], round_data["Opponent Move"], round_data["Outcome"], game.player_score_game, round_data["Reasoning"])


# Running the experiment:
if __name__ == "__main__":
    experiment = GuessExperiment()
    asyncio.run(experiment.run_experiment())
    print("Experiment completed. Results saved in", CSV_FILE_PATH)